# Copyright (c) 2011 Cloudera, Inc. All rights reserved.

"""
A helper to monitor directories.
"""

from cmf.monitor import schema
from cmf.monitor.host_dir_update import HostDirUpdate
from cmf.monitor.role_dir_update import RoleDirUpdate

class DirMonitor(object):
  def __init__(self, fs_map, path):
    self._fs_map = fs_map
    self._path = path
    self._fs = self._fs_map.get_filesystem_by_path(self._path)
    if self._fs is None:
      raise Exception("Could not find local file system for %s" % (self._path,))

    self._usage_info = None

  def collect(self):
    """Collect new metric values"""
    self._usage_info = self._fs_map.disk_usage(self._fs.mountpoint)

  def get_role_dir_update(self):
    update = RoleDirUpdate(self._path, self._fs.mountpoint)
    self._add_metrics(update)
    return update

  def get_host_dir_update(self):
    update = HostDirUpdate(self._path, self._fs.mountpoint)
    self._add_metrics(update)
    return update

  def _add_metrics(self, update):
    if self._usage_info is not None:
      update.add_metric(schema.FS_CAPACITY, self._usage_info['total'])
      update.add_metric(schema.FS_CAPACITY_FREE, self._usage_info['free'])



