#!/bin/bash
# Copyright (c) 2011-2012 Cloudera, Inc. All rights reserved.

set -x

# Time marker for both stderr and stdout
date; date 1>&2

cloudera_config=`dirname $0`
cloudera_config=`cd "$cloudera_config/../common"; pwd`
. ${cloudera_config}/cloudera-config.sh

locate_java_home

# We make the client configs world readable so that they can be used by
# any user logged into Hue.

export HUE_CONF_DIR=$CONF_DIR

export HADOOP_CONF_DIR=$CONF_DIR/hadoop-conf
# If Hue is using yarn, then hadoop conf files are in yarn-conf
if [[ -d $CONF_DIR/yarn-conf ]]; then
  export HADOOP_CONF_DIR=$CONF_DIR/yarn-conf
  export HADOOP_MAPRED_HOME=$CDH_MR2_HOME
fi
if [[ -d $HADOOP_CONF_DIR ]]; then
  chmod -R a+r $HADOOP_CONF_DIR
else
  echo "$HADOOP_CONF_DIR does not exist"
  exit 1
fi
export HIVE_CONF_DIR=$CONF_DIR/hive-conf
chmod -R a+r $HIVE_CONF_DIR

export HADOOP_HOME=$CDH_HADOOP_HOME
export HIVE_HOME=$CDH_HIVE_HOME
export HUE_HOME=$CDH_HUE_HOME

echo "HUE_HOME=$HUE_HOME"
echo "HUE_CONF_DIR=$HUE_CONF_DIR"
echo "HADOOP_HOME=$HADOOP_HOME"
echo "HADOOP_CONF_DIR=$HADOOP_CONF_DIR"
echo "HIVE_HOME=$HIVE_HOME"
echo "HIVE_DEFAULT_XML=$HIVE_DEFAULT_XML"
echo "HIVE_CONF_DIR=$HIVE_CONF_DIR"

HBASE_CONF_DIR=$CONF_DIR/hbase-conf
if [[ -d $HBASE_CONF_DIR ]]; then
  chmod -R a+r $HBASE_CONF_DIR
  echo "HBASE_CONF_DIR=$HBASE_CONF_DIR"
fi

# This must be writable by the hue user.
export PYTHON_EGG_CACHE=$HUE_CONF_DIR/.python-eggs

# local replacement of substitution variables.
# i.e. master doesn't know final resting place of configuration files
#   which may require absolute path (e.g. dfs_allows)
perl -pi -e "s#{{JAVA_HOME}}#$JAVA_HOME#g" $HUE_CONF_DIR/*.ini
perl -pi -e "s#{{HADOOP_CONF_DIR}}#$HADOOP_CONF_DIR#g" $HUE_CONF_DIR/*.ini
perl -pi -e "s#{{HUE_CONF_DIR}}#$HUE_CONF_DIR#g" $HUE_CONF_DIR/*.ini
if [[ -n $HBASE_CONF_DIR ]]; then
  perl -pi -e "s#{{HBASE_CONF_DIR}}#$HBASE_CONF_DIR#g" $HUE_CONF_DIR/*.ini
fi
if [ "4" = "$CDH_VERSION" ]; then
  perl -pi -e "s#{{HADOOP_HDFS_HOME}}#$CDH_HDFS_HOME#g" $HUE_CONF_DIR/*.ini
  perl -pi -e "s#{{HADOOP_MR1_HOME}}#$CDH_MR1_HOME#g" $HUE_CONF_DIR/*.ini
  perl -pi -e "s#{{HADOOP_MR2_HOME}}#$CDH_MR2_HOME#g" $HUE_CONF_DIR/*.ini
  perl -pi -e "s#{{HADOOP_BIN}}#$CDH_HADOOP_BIN#g" $HUE_CONF_DIR/*.ini
fi

# Hive, infuriatingly, does not provide
# a jar file with hive-default.xml in it, so
# we have to find it elsewhere and make
# sure it wanders in.
if [[ "$1" == beeswax_server || "$1" == runcpserver ]]; then
  echo Copying hive-default.xml to $HIVE_CONF_DIR
  cp -v "$HIVE_DEFAULT_XML" "$HIVE_CONF_DIR"
elif [[ $1 == kt_renewer ]]; then
  # The Kerberos ticket renewer role needs to know where kinit is.
  if [ -d /usr/kerberos/bin ]; then
    export PATH=/usr/kerberos/bin:$PATH
  fi
  KINIT_PATH=`which kinit`
  KINIT_PATH=${KINIT_PATH-/usr/bin/kinit}
  perl -pi -e "s#{{KINIT_PATH}}#$KINIT_PATH#g" $HUE_CONF_DIR/*.ini
fi

HUE="$HUE_HOME/build/env/bin/hue"
if [ "dumpdata" = "$1" ]; then
  umask 037
  echo Executing "$HUE $1 > $2"
  "$HUE" "$1" > "$2"
else 
  echo Executing "$HUE" "$@"
  exec "$HUE" "$@"
fi
