-- Copyright (c) 2012 Cloudera, Inc. All rights reserved.
--
-- Version 30 of the CMF schema. Migration code to be applied on top of the
-- previous version.
--

update SCHEMA_VERSION set VERSION=30;

--
-- Link services to clusters
--

alter table SERVICES
    add CLUSTER_ID number;

create index IDX_SERVICE_CLUSTER on SERVICES (CLUSTER_ID);

alter table SERVICES
    add constraint FK_SERVICE_CLUSTER
    foreign key (CLUSTER_ID)
    references CLUSTERS
    deferrable initially immediate;

alter table CLUSTERS
    add constraint CLUSTERS_NAME_UNIQUE unique (NAME);

alter table CLUSTERS
    modify NAME constraint NOT_NULL_CLUSTER_NAME not null;

--
-- The "default" cluster is created lazily so we have
-- to ensure its presence now.
--
-- 'from SERVICES' has the effect of only creating the
-- cluster if there is at least one service
--
-- The 'where' clause has the effect of only creating
-- the cluster if there is no cluster already
--

insert into CLUSTERS
    select 1, 'default'
    from DUAL
    where not exists (select 1 from CLUSTERS)
        and exists (select 1 from SERVICES);

update SERVICES s
    set CLUSTER_ID=(select min(CLUSTER_ID) from CLUSTERS)
    where SERVICE_TYPE!='MGMT';